from behave import when, then, step


class Hello():

    @step('we have behave installed')
    def step_impl_installation(context):
        pass

    @when('we implement a test')
    def step_impl_test(context):
        assert True is not False

    @then('behave will test it for us!')
    def step_impl_verify(context):
        assert context.failed is False



#!/usr/bin/env python

# Copyright (c) 2021 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Implement a Xtesting driver to run any Ansible playbook."""

import logging
import os
import shutil
import time

import ansible_runner

from xtesting.core import testcase


class Ansible(testcase.TestCase):
    """Class designed to run any Ansible playbook via ansible-runner."""

    __logger = logging.getLogger(__name__)

    def check_requirements(self):
        """Check if ansible-playbook is in $PATH"""
        self.is_skipped = not shutil.which("ansible-playbook")
        if self.is_skipped:
            self.__logger.warning("ansible-playbook is missing")

    def run(self, **kwargs):
        """ Wrap ansible_runner.interface.run()

        It calls ansible_runner.interface.run() by converting the testcase
        description data to kwargs. It only overrides quiet and artifact_dir to
        implement the Xtesting behavior.

        Following the playbook logic, criteria is considered as boolean
        whatever the value set in testcases.yaml.

        Args:
            kwargs: Arbitrary keyword arguments.

        Returns:
            EX_OK if the playbook ran well.
            EX_RUN_ERROR otherwise.
        """
        status = self.EX_RUN_ERROR
        self.start_time = time.time()
        if ("private_data_dir" in kwargs and
                os.path.isdir(kwargs['private_data_dir'])):
            try:
                if not os.path.exists(self.res_dir):
                    os.makedirs(self.res_dir)
                if "quiet" not in kwargs:
                    kwargs["quiet"] = True
                kwargs["artifact_dir"] = self.res_dir
                runner = ansible_runner.run(**kwargs)
                self.details = runner.stats
                if runner.rc == 0:
                    self.result = 100
                status = self.EX_OK
            except Exception:  # pylint: # pylint: disable=broad-except
                self.__logger.exception("Cannot execute the playbook")
        else:
            self.__logger.error(
                "Please set a relevant private_data_dir in testcases.yaml")
        self.stop_time = time.time()
        return status



#!/usr/bin/env python

# Copyright (c) 2017 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define the class required to fully cover decorators."""

from datetime import datetime
import errno
import json
import logging
import os
import unittest

import mock

from xtesting.core import testcase
from xtesting.utils import decorators

__author__ = "Cedric Ollivier <cedric.ollivier@orange.com>"

DIR = '/dev'
FILE = f'{DIR}/null'
URL = f'file://{FILE}'


class FakeTestCase(testcase.TestCase):
    # pylint: disable=missing-docstring

    def run(self, **kwargs):
        return testcase.TestCase.EX_OK


class DecoratorsTesting(unittest.TestCase):
    # pylint: disable=missing-docstring

    _case_name = 'base'
    _project_name = 'xtesting'
    _start_time = 1.0
    _stop_time = 2.0
    _result = 'PASS'
    _version = 'unknown'
    _build_tag = 'none'
    _node_name = 'bar'
    _deploy_scenario = 'foo'
    _installer_type = 'debian'

    def setUp(self):
        os.environ['INSTALLER_TYPE'] = self._installer_type
        os.environ['DEPLOY_SCENARIO'] = self._deploy_scenario
        os.environ['NODE_NAME'] = self._node_name
        os.environ['BUILD_TAG'] = self._build_tag

    def test_wraps(self):
        self.assertEqual(testcase.TestCase.push_to_db.__name__,
                         "push_to_db")

    def _get_json(self):
        stop_time = datetime.fromtimestamp(self._stop_time).strftime(
            '%Y-%m-%d %H:%M:%S')
        start_time = datetime.fromtimestamp(self._start_time).strftime(
            '%Y-%m-%d %H:%M:%S')
        data = {'project_name': self._project_name,
                'stop_date': stop_time, 'start_date': start_time,
                'case_name': self._case_name, 'build_tag': self._build_tag,
                'pod_name': self._node_name, 'installer': self._installer_type,
                'scenario': self._deploy_scenario, 'version': self._version,
                'details': {}, 'criteria': self._result}
        return json.dumps(data, sort_keys=True)

    def _get_testcase(self):
        test = FakeTestCase(
            project_name=self._project_name, case_name=self._case_name)
        test.start_time = self._start_time
        test.stop_time = self._stop_time
        test.result = 100
        test.details = {}
        return test

    def test_run_fake(self):
        test = self._get_testcase()
        self.assertEqual(test.run(), testcase.TestCase.EX_OK)

    @mock.patch('os.path.join')
    @mock.patch('re.sub')
    @mock.patch('requests.post')
    def test_http_shema(self, *args):
        os.environ['TEST_DB_URL'] = 'http://127.0.0.1'
        test = self._get_testcase()
        self.assertEqual(test.push_to_db(), testcase.TestCase.EX_OK)
        args[0].assert_called_once_with(
            'http://127.0.0.1', data=self._get_json(),
            headers={'Content-Type': 'application/json'})

    def test_wrong_shema(self):
        os.environ['TEST_DB_URL'] = '/dev/null'
        test = self._get_testcase()
        self.assertEqual(
            test.push_to_db(), testcase.TestCase.EX_PUSH_TO_DB_ERROR)

    def _test_dump(self):
        os.environ['TEST_DB_URL'] = URL
        with mock.patch.object(decorators, 'open', mock.mock_open(),
                               create=True) as mock_open:
            test = self._get_testcase()
            self.assertEqual(test.push_to_db(), testcase.TestCase.EX_OK)
        mock_open.assert_called_once_with(FILE, 'a', encoding='utf-8')
        handle = mock_open()
        call_args, _ = handle.write.call_args
        self.assertIn('POST', call_args[0])
        self.assertIn(self._get_json(), call_args[0])

    @mock.patch('os.makedirs')
    def test_default_dump(self, mock_method=None):
        self._test_dump()
        mock_method.assert_called_once_with(DIR)

    @mock.patch('os.makedirs', side_effect=OSError(errno.EEXIST, ''))
    def test_makedirs_dir_exists(self, mock_method=None):
        self._test_dump()
        mock_method.assert_called_once_with(DIR)

    @mock.patch('os.makedirs', side_effect=OSError)
    def test_makedirs_exc(self, *args):
        os.environ['TEST_DB_URL'] = URL
        test = self._get_testcase()
        self.assertEqual(
            test.push_to_db(), testcase.TestCase.EX_PUSH_TO_DB_ERROR)
        args[0].assert_called_once_with(DIR)


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)



#!/usr/bin/env python

# Copyright (c) 2018 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import os
import textwrap

import prettytable

INPUTS = {
    'CI_LOOP': 'daily',
    'DEBUG': 'false',
    'DEPLOY_SCENARIO': 'os-nosdn-nofeature-noha',
    'INSTALLER_TYPE': 'unknown',
    'BUILD_TAG': None,
    'NODE_NAME': None,
    'TEST_DB_URL': 'http://testresults.opnfv.org/test/api/v1/results',
    'TEST_DB_EXT_URL': None,
    'S3_ENDPOINT_URL': None,
    'S3_DST_URL': None,
    'HTTP_DST_URL': None
}


def get(env_var):
    # defaults to None if env_var is not found
    return os.environ.get(env_var, INPUTS.get(env_var))


def string():
    msg = prettytable.PrettyTable(
        header_style='upper', padding_width=5,
        field_names=['env var', 'value'])
    for env_var in INPUTS:
        msg.add_row([env_var, textwrap.fill(get(env_var), width=50) if get(
            env_var) else ''])
    return msg



#!/usr/bin/env python

# Copyright (c) 2019 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

"""Define classes required to run any Behave test suites."""

from __future__ import division

import logging
import os
import time

import json

from behave.__main__ import main as behave_main

from xtesting.core import testcase

__author__ = "Deepak Chandella <deepak.chandella@orange.com>"


class BehaveFramework(testcase.TestCase):
    """BehaveFramework runner."""
    # pylint: disable=too-many-instance-attributes

    __logger = logging.getLogger(__name__)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.json_file = os.path.join(self.res_dir, 'output.json')
        self.total_tests = 0
        self.pass_tests = 0
        self.fail_tests = 0
        self.skip_tests = 0
        self.response = None

    def parse_results(self):
        """Parse output.json and get the details in it."""
        with open(self.json_file, encoding='utf-8') as stream_:
            self.response = json.load(stream_)
            if self.response:
                self.total_tests = len(self.response)
            for item in self.response:
                if item['status'] == 'passed':
                    self.pass_tests += 1
                elif item['status'] == 'failed':
                    self.fail_tests += 1
                elif item['status'] == 'skipped':
                    self.skip_tests += 1
            self.result = 100 * (
                self.pass_tests / self.total_tests)
            self.details = {}
            self.details['total_tests'] = self.total_tests
            self.details['pass_tests'] = self.pass_tests
            self.details['fail_tests'] = self.fail_tests
            self.details['skip_tests'] = self.skip_tests
            self.details['tests'] = self.response

    def run(self, **kwargs):
        """Run the BehaveFramework feature files

        Here are the steps:
           * create the output directories if required,
           * run behave features with parameters
           * get the results in output.json,

        Args:
            kwargs: Arbitrary keyword arguments.

        Returns:
            EX_OK if all suites ran well.
            EX_RUN_ERROR otherwise.
        """
        try:
            suites = kwargs["suites"]
        except KeyError:
            self.__logger.exception("Mandatory args were not passed")
            return self.EX_RUN_ERROR
        if not os.path.exists(self.res_dir):
            try:
                os.makedirs(self.res_dir)
            except Exception:  # pylint: disable=broad-except
                self.__logger.exception("Cannot create %s", self.res_dir)
                return self.EX_RUN_ERROR
        config = ['--junit', f'--junit-directory={self.res_dir}',
                  '--format=json', f'--outfile={self.json_file}']
        html_file = os.path.join(self.res_dir, 'output.html')
        config += ['--format=behave_html_formatter:HTMLFormatter',
                   f'--outfile={html_file}']
        if kwargs.get("tags", False):
            config += ['--tags='+','.join(kwargs.get("tags", []))]
        if kwargs.get("console", False):
            config += ['--format=pretty', '--outfile=-']
        for feature in suites:
            config.append(feature)
        self.start_time = time.time()
        behave_main(config)
        self.stop_time = time.time()

        try:
            self.parse_results()
            self.__logger.info("Results were successfully parsed")
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("Cannot parse results")
            return self.EX_RUN_ERROR
        return self.EX_OK



#!/usr/bin/env python

# Copyright (c) 2020 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=too-many-instance-attributes

"""Implement a Xtesting driver to run mts suite."""

import csv
import logging
import os
import shutil
import time

from lxml import etree
import prettytable

from xtesting.core import feature
from xtesting.core import testcase


__author__ = ("Vincent Mahe <v.mahe@orange.com>, "
              "Cedric Ollivier <cedric.ollivier@orange.com>")


class MTSLauncher(feature.BashFeature):
    """Class designed to run MTS tests."""

    __logger = logging.getLogger(__name__)
    mts_install_dir = "/opt/mts"

    def check_requirements(self):
        """Check if startCmd.sh is in /opt/mts/bin"""
        if not os.path.exists(
                os.path.join(self.mts_install_dir, 'bin/startCmd.sh')):
            self.__logger.warning(
                "mts is not available for arm for the time being")
            self.is_skipped = True

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # Location of the HTML report generated by MTS
        self.mts_stats_dir = os.path.join(self.res_dir, 'mts_stats_report')
        # Location of the log files generated by MTS for each test.
        # Need to end path with a separator because of a bug in MTS.
        self.mts_logs_dir = os.path.join(self.res_dir,
                                         'mts_logs' + os.path.sep)
        # The location of file named testPlan.csv
        # that it always in $MTS_HOME/logs
        self.mts_result_csv_file = self.mts_install_dir + os.path.sep
        self.mts_result_csv_file += ("logs" + os.path.sep + "testPlan.csv")
        self.total_tests = 0
        self.pass_tests = 0
        self.fail_tests = 0
        self.skip_tests = 0
        self.response = None
        self.testcases = []

    def parse_results(self):
        """Parse testPlan.csv containing the status of each testcase of the test file.
        See sample file in `xtesting/samples/mts/output/testPlan.csv`
        """
        with open(self.mts_result_csv_file, encoding='utf-8') as stream_:
            self.__logger.info("Parsing file : %s", self.mts_result_csv_file)
            reader = csv.reader(stream_, delimiter=';')
            rownum = 0
            _tests_data = []
            msg = prettytable.PrettyTable(
                header_style='upper', padding_width=5,
                field_names=['MTS test', 'MTS test case',
                             'status'])
            for row in reader:
                _test_dict = {}
                nb_values = len(row)
                if rownum > 0:
                    # If there's only one delimiter,
                    # it is the name of the <test> elt
                    if nb_values == 2:
                        test_name = row[0]
                        _test_dict['parent'] = test_name
                    elif nb_values == 3:
                        testcase_name = row[0].lstrip()
                        testcase_status = row[2]
                        self.total_tests += 1
                        if testcase_status == 'OK':
                            self.pass_tests += 1
                        elif testcase_status == 'Failed':
                            self.fail_tests += 1
                        elif testcase_status == '?':
                            self.skip_tests += 1
                        _test_dict['status'] = testcase_status
                        _test_dict['name'] = testcase_name
                        msg.add_row(
                            [test_name,
                             _test_dict['name'],
                             _test_dict['status']])
                rownum += 1
                _tests_data.append(_test_dict)
            try:
                self.result = 100 * (
                    self.pass_tests / self.total_tests)
            except ZeroDivisionError:
                self.__logger.error("No test has been run")
            self.__logger.info("MTS Test result:\n\n%s\n", msg.get_string())
            self.details = {}
            self.details['description'] = "Execution of some MTS tests"
            self.details['total_tests'] = self.total_tests
            self.details['pass_tests'] = self.pass_tests
            self.details['fail_tests'] = self.fail_tests
            self.details['skip_tests'] = self.skip_tests
            self.details['tests'] = _tests_data

    def parse_xml_test_file(self, xml_test_file):
        """Parse the XML file containing the test definition for MTS.
        See sample file in `xtesting/samples/mts/test.xml`
        """
        nb_testcases = -1
        self.__logger.info(
            "Parsing XML test file %s containing the MTS tests definitions.",
            xml_test_file)
        try:
            parser = etree.XMLParser(load_dtd=True, resolve_entities=True)
            self.__logger.info("XML test file %s successfully parsed.",
                               xml_test_file)
            root = etree.parse(xml_test_file, parser=parser)
            # Need to look at all child nodes because there may be
            # some <for> elt between <test> and <testcase> elt
            self.testcases = root.xpath('//test//testcase/@name')
            nb_testcases = len(self.testcases)
            if nb_testcases == 0:
                self.__logger.warning("Found no MTS testcase !")
            elif nb_testcases == 1:
                self.__logger.info("Found only one MTS testcase: %s",
                                   self.testcases[0])
            else:
                self.__logger.info("Found %d MTS testcases :", nb_testcases)
                for mts_testcase in self.testcases:
                    self.__logger.info("    - %s", mts_testcase)
        except etree.XMLSyntaxError as xml_err:
            self.__logger.error("Error while parsing XML test file: %s",
                                str(xml_err))
        return nb_testcases

    def check_enabled_mts_test_cases(self, enabled_testcases):
        """Make sure that all required MTS test cases exist
        in the XML test file.
        """
        if enabled_testcases:
            # Verify if the MTS test case exists in the whole list of test
            # cases declared in the test XML file
            for enabled_testcase in enabled_testcases:
                if enabled_testcase not in self.testcases:
                    self.__logger.error(
                        "The required MTS testcase named `%s` does not exist"
                        " !", enabled_testcase)
                    return False
        return True

    def execute(self, **kwargs):  # pylint: disable=too-many-locals
        try:
            # Read specific parameters for MTS
            test_file = kwargs["test_file"]
            log_level = kwargs[
                "log_level"] if "log_level" in kwargs else "INFO"
            store_method = kwargs[
                "store_method"] if "store_method" in kwargs else "FILE"
            # Must use the $HOME_MTS/bin as current working dir
            cwd = self.mts_install_dir + os.path.sep + "bin"

            # Get the list of enabled MTS testcases, if any
            enabled_testcases = kwargs[
                "testcases"] if "testcases" in kwargs else []
            enabled_testcases_str = ''
            if enabled_testcases:
                enabled_testcases_str = ' '.join(enabled_testcases)
                check_ok = self.check_enabled_mts_test_cases(enabled_testcases)
                if not check_ok:
                    return -3

            # Build command line to launch for MTS
            cmd = (f"cd {cwd} && ./startCmd.sh {test_file} "
                   f"{enabled_testcases_str} -sequential -levelLog:{log_level}"
                   f" -storageLog:{store_method}"
                   f" -config:stats.REPORT_DIRECTORY+{self.mts_stats_dir}"
                   f" -config:logs.STORAGE_DIRECTORY+{self.mts_logs_dir}"
                   " -genReport:true"
                   " -showRep:false")

            # Make sure to create the necessary output sub-folders for MTS
            # and cleanup output files from previous run.
            if os.path.exists(self.mts_result_csv_file):
                os.remove(self.mts_result_csv_file)

            if os.path.isdir(self.mts_stats_dir):
                shutil.rmtree(self.mts_stats_dir)
            os.makedirs(self.mts_stats_dir)

            if os.path.isdir(self.mts_logs_dir):
                shutil.rmtree(self.mts_logs_dir)
            os.makedirs(self.mts_logs_dir)

            self.__logger.info(
                "MTS statistics output dir: %s ", self.mts_stats_dir)
            self.__logger.info("MTS logs output dir: %s ", self.mts_logs_dir)

            kwargs.pop("cmd", None)
            return super().execute(cmd=cmd, **kwargs)

        except KeyError:
            self.__logger.error("Missing mandatory arg for MTS. kwargs: %s",
                                kwargs)
        return -1

    def run(self, **kwargs):
        """Runs the MTS suite"""
        self.start_time = time.time()
        exit_code = testcase.TestCase.EX_RUN_ERROR
        self.result = 0
        try:
            nb_testcases = self.parse_xml_test_file(kwargs["test_file"])
            # Do something only if there are some MTS test cases in the test
            # file
            if nb_testcases > 0:
                if self.execute(**kwargs) == 0:
                    exit_code = testcase.TestCase.EX_OK
                    try:
                        self.parse_results()
                    except Exception:  # pylint: disable=broad-except
                        self.__logger.exception(
                            "Cannot parse result file "
                            "$MTS_HOME/logs/testPlan.csv")
                        exit_code = testcase.TestCase.EX_RUN_ERROR
        except Exception:  # pylint: disable=broad-except
            self.__logger.exception("%s FAILED", self.project_name)
        self.stop_time = time.time()
        return exit_code



#!/usr/bin/env python

# Copyright (c) 2018 Orange and others.
#
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Apache License, Version 2.0
# which accompanies this distribution, and is available at
# http://www.apache.org/licenses/LICENSE-2.0

# pylint: disable=missing-docstring

import logging
import os
import unittest
import importlib

from xtesting.utils import env


class EnvTesting(unittest.TestCase):
    # pylint: disable=missing-docstring

    def setUp(self):
        os.environ['FOO'] = 'foo'
        os.environ['BUILD_TAG'] = 'master'
        os.environ['CI_LOOP'] = 'weekly'

    def test_get_unset_unknown_env(self):
        del os.environ['FOO']
        self.assertEqual(env.get('FOO'), None)

    def test_get_unknown_env(self):
        self.assertEqual(env.get('FOO'), 'foo')
        importlib.reload(env)

    def test_get_unset_env(self):
        del os.environ['CI_LOOP']
        self.assertEqual(
            env.get('CI_LOOP'), env.INPUTS['CI_LOOP'])

    def test_get_env(self):
        self.assertEqual(
            env.get('CI_LOOP'), 'weekly')

    def test_get_unset_env2(self):
        del os.environ['BUILD_TAG']
        self.assertEqual(
            env.get('BUILD_TAG'), env.INPUTS['BUILD_TAG'])

    def test_get_env2(self):
        self.assertEqual(env.get('BUILD_TAG'), 'master')


if __name__ == "__main__":
    logging.disable(logging.CRITICAL)
    unittest.main(verbosity=2)
